
package org.objectweb.fractal.bf.connectors.tcp;

import java.net.Inet4Address;
import java.net.UnknownHostException;

import org.objectweb.fractal.bf.AbstractExportId;
import org.objectweb.fractal.bf.BindingFactoryException;

/**
 * An export identifier for TCP/Dream based connectors.
 * 
 * @author Valerio Schiavoni <valerio.schiavoni@gmail.com>
 */
public class TcpConnectorExportId extends AbstractExportId
{

  /**
   * A generated serial versionUID, required as this class is serializable.
   */
  private static final long serialVersionUID = -5267834080159527416L;

  String                    port;

  String                    hostAddress;

  /**
   * @param itfName the name of the interface
   * @param componentOwnerName the name of the owner component
   * @param listenPort
   * @param serverItf the reference
   */
  public TcpConnectorExportId(String itfName, String componentOwnerName,
      String listenPort) throws BindingFactoryException
  {

    super(itfName, componentOwnerName);
    this.port = listenPort;

    try
    {
      // if there are multiple network interfaces
      this.hostAddress = Inet4Address.getLocalHost().getHostAddress();
    }
    catch (UnknownHostException e)
    {
      e.printStackTrace();
    }

  }

  /**
   * @return the port
   */
  public String getPort()
  {
    return port;
  }

  /**
   * @return the hostAddress
   */
  public String getHostAddress()
  {
    return hostAddress;
  }

  /**
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString()
  {
    return "[TcpConnectorExportId] Address: " + this.hostAddress + " Port: "
        + this.port;
  }

  /**
   * @see org.objectweb.fractal.bf.ExportId#getExportMode()
   */
  public String getExportMode()
  {
    return "tcp";
  }


}
